import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { Observable, of } from 'rxjs';
import { CommandResource } from './command.model';
import { tapOnCommandSuccessfullyDone } from './command.rxjs.operator';

import * as CommandUtil from './command.util';

describe('Command rxjs operator', () => {
  describe('tapOnCommandSuccessfullyDone', () => {
    const command = createStateResource(createCommandResource());
    const commandStateResource$: Observable<StateResource<CommandResource>> = of(command);

    it('should call the execute the runnable if command is successfully done', (done) => {
      jest.spyOn(CommandUtil, 'isSuccessfulDone').mockReturnValue(true);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnCommandSuccessfullyDone(runnable)).subscribe(() => {
        expect(runnable).toHaveBeenCalledWith(command);
        done();
      });
    });

    it('should NOT execute the runnable if command is not done', (done) => {
      jest.spyOn(CommandUtil, 'isSuccessfulDone').mockReturnValue(false);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnCommandSuccessfullyDone(runnable)).subscribe(() => {
        expect(runnable).not.toHaveBeenCalled();
        done();
      });
    });
  });
});
