import { Injectable } from '@angular/core';
import {
  COMMAND_ERROR_MESSAGES,
  CREATE_COMMAND_MESSAGE_BY_ORDER,
  CommandErrorMessage,
  CommandProps,
  LOESCH_ANFORDERUNG_ZURUECKNEHMEN_CONCURRENT_TO_DELETE_VORGANG_MESSAGE,
} from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { getApiErrorFromHttpErrorResponse, isNotFound } from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { VorgangWithEingangAction, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { hasLink } from '@ngxp/rest';
import { of } from 'rxjs';
import { catchError, filter, map, switchMap, tap } from 'rxjs/operators';
import { isVorgangLoeschenCommand } from '../loesch-anforderung.util';
import { LoeschAnforderungZuruecknehmenFailedAndReloadVorgangFailureProps } from './loesch-anforderung.actions';
import { LoeschAnforderungRepository } from './loesch-anforderung.repository';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as VorgangActions from '../../../../vorgang-shared/src/lib/+state/vorgang.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';

@Injectable()
export class LoeschAnforderungEffects {
  constructor(
    private readonly actions$: Actions,
    private repository: LoeschAnforderungRepository,
    private snackBarService: SnackBarService,
    private navigationService: NavigationService,
  ) {}

  loadLoeschAnforderung$ = createEffect(() =>
    this.actions$.pipe(
      ofType(VorgangActions.loadVorgangWithEingangSuccess),
      filter((props: VorgangWithEingangAction) =>
        hasLink(props.vorgangWithEingang, VorgangWithEingangLinkRel.LOESCH_ANFORDERUNG),
      ),
      switchMap((props: VorgangWithEingangAction) =>
        this.repository.getLoeschAnforderung(props.vorgangWithEingang).pipe(
          map((loeschAnforderung) =>
            LoeschAnforderungActions.loadLoeschAnforderungSuccess({ loeschAnforderung }),
          ),
          catchError((error) =>
            of(
              LoeschAnforderungActions.loadLoeschAnforderungFailure({
                apiError: getApiErrorFromHttpErrorResponse(error),
              }),
            ),
          ),
        ),
      ),
    ),
  );

  showVorgangLoeschenSuccessSnackbar$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(CommandActions.createCommandSuccess),
        filter((props: CommandProps) => isVorgangLoeschenCommand(props.command.order)),
        tap((props: CommandProps) =>
          this.snackBarService.show(
            props.command,
            CREATE_COMMAND_MESSAGE_BY_ORDER[props.command.order],
          ),
        ),
      ),
    { dispatch: false },
  );

  loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(
          LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangSuccess,
        ),
        tap(() =>
          this.snackBarService.showError(
            COMMAND_ERROR_MESSAGES[CommandErrorMessage.CONCURRENT_MODIFICATION],
          ),
        ),
      ),
    { dispatch: false },
  );

  loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(
          LoeschAnforderungActions.loeschAnforderungZuruecknehmenFailedAndReloadVorgangFailure,
        ),
        filter((props: LoeschAnforderungZuruecknehmenFailedAndReloadVorgangFailureProps) =>
          isNotFound(props.error.error.status),
        ),
        tap((props: LoeschAnforderungZuruecknehmenFailedAndReloadVorgangFailureProps) => {
          this.snackBarService.showInfo(
            LOESCH_ANFORDERUNG_ZURUECKNEHMEN_CONCURRENT_TO_DELETE_VORGANG_MESSAGE,
          );
          this.navigationService.navigate(props.overviewUrl);
        }),
      ),
    { dispatch: false },
  );
}
