import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  LoeschAnforderungLinkRel,
  LoeschAnforderungResource,
  LoeschAnforderungService,
} from '@alfa-client/loesch-anforderung-shared';
import {
  HasLinkPipe,
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createLoeschAnforderungResource } from '../../../../loesch-anforderung-shared/test/loesch-anforderung';
import { LoeschAnforderungZuruecknehmenButtonContainerComponent } from './loesch-anforderung-zuruecknehmen-button-container.component';
import { LoeschAnforderungZuruecknehmenButtonComponent } from './loesch-anforderung-zuruecknehmen-button/loesch-anforderung-zuruecknehmen-button.component';

describe('LoeschAnforderungZuruecknehmenButtonComponent', () => {
  let component: LoeschAnforderungZuruecknehmenButtonContainerComponent;
  let fixture: ComponentFixture<LoeschAnforderungZuruecknehmenButtonContainerComponent>;

  const service: Mock<LoeschAnforderungService> = mock(LoeschAnforderungService);
  const loeschAnforderungResource: StateResource<LoeschAnforderungResource> =
    createEmptyStateResource();

  const loeschAnforderungZuruecknehmenButtonComponent: string = getDataTestIdOf(
    'loesch-anforderung-zuruecknehmen-button-component',
  );

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        LoeschAnforderungZuruecknehmenButtonContainerComponent,
        HasLinkPipe,
        MockComponent(LoeschAnforderungZuruecknehmenButtonComponent),
      ],
      providers: [
        {
          provide: LoeschAnforderungService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(LoeschAnforderungZuruecknehmenButtonContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should get loeschAnforderungZuruecknehmenCommand', () => {
      component.ngOnInit();

      expect(service.getLoschAnforderungZuruecknehmenCommand).toHaveBeenCalled();
    });

    it('should get LoeschAnforderung Resource', () => {
      component.ngOnInit();

      expect(service.getLoeschAnforderung).toHaveBeenCalled();
    });
  });

  describe('loeschAnforderungZuruecknehmen', () => {
    it('should call service', () => {
      component.loeschAnforderungZuruecknehmen(loeschAnforderungResource);

      expect(service.zuruecknehmen).toHaveBeenCalledWith(loeschAnforderungResource.resource);
    });
  });

  describe('loeschAnforderungButtonComponent', () => {
    it('should be visible if link exists on LoeschAnforderung', () => {
      component.loeschAnforderungStateResource$ = of(
        createStateResource(
          createLoeschAnforderungResource([LoeschAnforderungLinkRel.ZURUECKNEHMEN]),
        ),
      );
      fixture.detectChanges();

      existsAsHtmlElement(fixture, loeschAnforderungZuruecknehmenButtonComponent);
    });

    it('should be hidden if link NOT exists', () => {
      component.loeschAnforderungStateResource$ = of(
        createStateResource(createLoeschAnforderungResource()),
      );
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, loeschAnforderungZuruecknehmenButtonComponent);
    });
  });
});
