import { ComponentFixture, TestBed } from '@angular/core/testing';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import {
  IconButtonWithSpinnerComponent,
  OzgcloudStrokedButtonWithSpinnerComponent,
} from '@alfa-client/ui';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { LoeschenAnfordernButtonComponent } from './loeschen-anfordern-button.component';

describe('LoeschenAnfordernButtonComponent', () => {
  let component: LoeschenAnfordernButtonComponent;
  let fixture: ComponentFixture<LoeschenAnfordernButtonComponent>;

  const loeschenAnfordernButton: string = getDataTestIdOf('loeschen-anfordern-button');
  const loeschenAnfordernIconButton: string = getDataTestIdOf('loeschen-anfordern-icon-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        LoeschenAnfordernButtonComponent,
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
        MockComponent(IconButtonWithSpinnerComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(LoeschenAnfordernButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('show as icon button', () => {
    beforeEach(() => {
      component.showAsIconButton = true;
      fixture.detectChanges();
    });

    it('should hide button', () => {
      notExistsAsHtmlElement(fixture, loeschenAnfordernButton);
    });

    it('should show iconButton', () => {
      existsAsHtmlElement(fixture, loeschenAnfordernIconButton);
    });
  });

  describe('show as button', () => {
    beforeEach(() => {
      component.showAsIconButton = false;
      fixture.detectChanges();
    });

    it('should show button', () => {
      existsAsHtmlElement(fixture, loeschenAnfordernButton);
    });

    it('should hide iconButton visible', () => {
      notExistsAsHtmlElement(fixture, loeschenAnfordernIconButton);
    });
  });
});
