/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { EMPTY_ARRAY, createStateResource } from '@alfa-client/tech-shared';
import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { PostfachMailListLinkRel } from '../src/lib/postfach.linkrel';
import { PostfachNachrichtMessageCode } from '../src/lib/postfach.message-code';
import {
  Direction,
  PostfachFeatures,
  PostfachMail,
  PostfachMailFormDialogData,
  PostfachMailListResource,
  PostfachMailResource,
  PostfachSettings,
  ReplyOption,
} from '../src/lib/postfach.model';

export function createPostfachMail(): PostfachMail {
  return {
    subject: faker.random.words(3),
    createdAt: faker.date.past(),
    direction: Direction.OUT,
    mailBody: faker.random.words(30),
    replyOption: ReplyOption.POSSIBLE,
    sentAt: faker.date.past(),
    sentSuccessful: false,
    messageCode: PostfachNachrichtMessageCode.CONNECTION_FAILED,
    attachments: faker.datatype.uuid(),
  };
}

export function createPostfachMailResource(linkRelations: string[] = []): PostfachMailResource {
  return toResource(createPostfachMail(), linkRelations);
}

export function createPostfachMailResources(linkRelations: string[] = []): PostfachMailResource[] {
  return times(10, () => toResource(createPostfachMailResource(), [...linkRelations]));
}

export function createPostfachMailListResource(
  linkRelations: string[] = [],
): PostfachMailListResource {
  return toResource(createPostfachMailList(), [...linkRelations], {
    [PostfachMailListLinkRel.POSTFACH_MAIL_LIST]: createPostfachMailResources(),
  });
}

function createPostfachMailList(): Resource {
  return toResource({ features: createPostfachFeatures(), settings: createPostfachSettings() }, [
    'sendPostfachMail',
  ]);
}

export function createPostfachFeatures(): PostfachFeatures {
  return { reply: true };
}

export function createPostfachSettings(): PostfachSettings {
  return { signatur: 'Best regards' };
}

export function createEmptyPostfachSettings(): PostfachSettings {
  return { signatur: '' };
}

export class PostfachTestFactory {
  public static POSTFACH_NACHRICHT: PostfachMail = {
    subject: faker.random.words(3),
    createdAt: faker.date.past(),
    direction: Direction.OUT,
    mailBody: faker.random.words(30),
    replyOption: ReplyOption.POSSIBLE,
    sentAt: faker.date.past(),
    sentSuccessful: false,
    messageCode: PostfachNachrichtMessageCode.CONNECTION_FAILED,
    attachments: faker.datatype.uuid(),
  };

  public static POSTFACH_NACHRICHT_RESOURCE: PostfachMailResource = toResource(
    PostfachTestFactory.POSTFACH_NACHRICHT,
    EMPTY_ARRAY,
  );
  public static POSTFACH_NACHRICHT_LIST_RESOURCE = createPostfachMailListResource();
  public static POSTFACH_NACHRICHT_LIST_STATE_RESOURCE = createStateResource(
    PostfachTestFactory.POSTFACH_NACHRICHT_LIST_RESOURCE,
  );

  public static POSTFACH_MAIL_FORM_DIALOG_DATA_TITLE: string = '';
  public static POSTFACH_MAIL_FORM_DIALOG_DATA_EMPFAENGER: string = '';
  public static POSTFACH_MAIL_FORM_DIALOG_DATA_COMPONENT = null;

  public static POSTFACH_MAIL_FORM_DIALOG_DATA: PostfachMailFormDialogData = {
    postfachNachricht: PostfachTestFactory.POSTFACH_NACHRICHT_RESOURCE,
    empfaenger: PostfachTestFactory.POSTFACH_MAIL_FORM_DIALOG_DATA_EMPFAENGER,
    component: PostfachTestFactory.POSTFACH_MAIL_FORM_DIALOG_DATA_COMPONENT,
    postfachMailListStateResource: PostfachTestFactory.POSTFACH_NACHRICHT_LIST_STATE_RESOURCE,
    title: PostfachTestFactory.POSTFACH_MAIL_FORM_DIALOG_DATA_TITLE,
  };
}
