import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, FormGroup, FormGroupDirective, ReactiveFormsModule } from '@angular/forms';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import {
  FORBIDDEN_REPLY_OPTION_ITEM,
  POSSIBLE_REPLY_OPTION_ITEM,
  PostfachService,
} from '@alfa-client/postfach-shared';
import {
  Mock,
  existsAsHtmlElement,
  getMockComponent,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { PostfachMailFormservice } from '../postfach-mail.formservice';
import { PostfachNachrichtReplyEditorContainerComponent } from './postfach-nachricht-reply-editor-container.component';
import { CheckboxEnumEditorComponent } from '@alfa-client/ui';

describe('PostfachNachrichtReplyEditorContainerComponent', () => {
  let component: PostfachNachrichtReplyEditorContainerComponent;
  let fixture: ComponentFixture<PostfachNachrichtReplyEditorContainerComponent>;

  const replyOptionEditor: string = getDataTestIdOf('postfach-reply-option');

  const service: Mock<PostfachService> = mock(PostfachService);

  const formGroupDirective: FormGroupDirective = new FormGroupDirective([], []);
  const formGroup: FormGroup = new FormGroup({
    [PostfachMailFormservice.FIELD_REPLY_OPTION]: new FormControl(''),
  });

  beforeEach(async () => {
    formGroupDirective.form = formGroup;

    await TestBed.configureTestingModule({
      declarations: [
        PostfachNachrichtReplyEditorContainerComponent,
        MockComponent(CheckboxEnumEditorComponent),
      ],
      providers: [
        {
          provide: PostfachService,
          useValue: service,
        },
        {
          provide: FormGroupDirective,
          useValue: formGroupDirective,
        },
      ],
      imports: [ReactiveFormsModule, BrowserAnimationsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(PostfachNachrichtReplyEditorContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should get list stateResource', () => {
      component.ngOnInit();

      expect(service.getFeatures).toHaveBeenCalled();
    });
  });

  describe('replyOptionEditor', () => {
    describe('on available feature', () => {
      beforeEach(() => {
        component.postfachFeatures$ = of({ reply: true });
      });

      it('should be visible', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, replyOptionEditor);
      });

      it.skip('should be called with formControlName', () => {
        fixture.detectChanges();

        const enumEditor: CheckboxEnumEditorComponent = getMockComponent(
          fixture,
          CheckboxEnumEditorComponent,
        );
        expect(enumEditor.control).toBe(PostfachMailFormservice.FIELD_REPLY_OPTION);
      });

      it('should be called with defaultItem', () => {
        fixture.detectChanges();

        const enumEditor: CheckboxEnumEditorComponent = getMockComponent(
          fixture,
          CheckboxEnumEditorComponent,
        );
        expect(enumEditor.defaultItem).toBe(POSSIBLE_REPLY_OPTION_ITEM);
      });

      it('should be called with checked item', () => {
        fixture.detectChanges();

        const enumEditor: CheckboxEnumEditorComponent = getMockComponent(
          fixture,
          CheckboxEnumEditorComponent,
        );
        expect(enumEditor.checkedItem).toBe(POSSIBLE_REPLY_OPTION_ITEM);
      });

      it('should be called with unchecked item', () => {
        fixture.detectChanges();

        const enumEditor: CheckboxEnumEditorComponent = getMockComponent(
          fixture,
          CheckboxEnumEditorComponent,
        );
        expect(enumEditor.uncheckedItem).toBe(FORBIDDEN_REPLY_OPTION_ITEM);
      });
    });

    describe('on unavailable feature', () => {
      beforeEach(() => {
        component.postfachFeatures$ = of({ reply: false });
      });

      it('should be hidden', () => {
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, replyOptionEditor);
      });
    });
  });
});
