/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { UntypedFormBuilder } from '@angular/forms';
import { createStateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { UserProfileListResource, UserProfileService } from '@alfa-client/user-profile-shared';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createUserProfileListResource } from 'libs/user-profile-shared/test/user-profile';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { UserProfileSearchContainerComponent } from './user-profile-search-container.component';
import { UserProfileSearchComponent } from './user-profile-search/user-profile-search.component';
import { UserProfileSearchFormService } from './user-profile-search/user-profile.search.formservice';

describe('UserProfileSearchContainerComponent', () => {
  let component: UserProfileSearchContainerComponent;
  let fixture: ComponentFixture<UserProfileSearchContainerComponent>;

  const userProfileSearchFormService = mock(UserProfileSearchFormService);

  const userProfileListResource: UserProfileListResource = createUserProfileListResource();
  const userProfileService = {
    ...mock(UserProfileService),
    getSearchedUserProfiles: () => of(userProfileListResource),
  };

  const vorgangService = mock(VorgangService);

  beforeEach(async () => {
    TestBed.configureTestingModule({
      declarations: [
        UserProfileSearchContainerComponent,
        MockComponent(SpinnerComponent),
        MockComponent(UserProfileSearchComponent),
      ],
      providers: [
        UntypedFormBuilder,
        {
          provide: UserProfileSearchFormService,
          useValue: userProfileSearchFormService,
        },
        {
          provide: UserProfileService,
          useValue: userProfileService,
        },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    });

    vorgangService.getAssignUserCommand.mockReturnValue(
      of(createStateResource(createCommandResource())),
    );
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(UserProfileSearchContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });
});
