/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, notHasLink, StateResource } from '@alfa-client/tech-shared';
import { VorgangCommandService, VorgangWithEingangLinkRel, VorgangWithEingangResource, } from '@alfa-client/vorgang-shared';
import { Component, Input, OnInit } from '@angular/core';
import { hasLink } from '@ngxp/rest';
import { Observable, of } from 'rxjs';

@Component({
  selector: 'alfa-abschliessen-button',
  templateUrl: './abschliessen-button.component.html',
  styleUrls: ['./abschliessen-button.component.scss'],
})
export class AbschliessenButtonComponent implements OnInit {
  @Input() set vorgang(vorgang: VorgangWithEingangResource) {
    this.vorgangWithEingangResource = vorgang;
    this.isVisible =
      hasLink(vorgang, VorgangWithEingangLinkRel.ABSCHLIESSEN) &&
      notHasLink(vorgang, VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT) &&
      notHasLink(vorgang, VorgangWithEingangLinkRel.BESCHEID_DRAFT);
  }

  @Input() showAsIconButton: boolean = false;

  public isVisible: boolean = false;
  vorgangWithEingangResource: VorgangWithEingangResource;

  commandStateResource$: Observable<StateResource<CommandResource>> = of(
    createEmptyStateResource<CommandResource>(),
  );

  readonly linkRel = VorgangWithEingangLinkRel;

  constructor(private vorgangCommandService: VorgangCommandService) {}

  ngOnInit(): void {
    this.commandStateResource$ = this.vorgangCommandService.getAbschliessenCommand();
  }

  public abschliessen(): void {
    this.commandStateResource$ = this.vorgangCommandService.abschliessen(
      this.vorgangWithEingangResource,
    );
  }
}
