/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { EndgueltigLoeschenButtonContainerComponent, LoeschAnforderungZuruecknehmenButtonContainerComponent, LoeschenAnfordernButtonContainerComponent } from '@alfa-client/loesch-anforderung';
import { PostfachMailButtonContainerComponent } from '@alfa-client/postfach';
import { HasLinkPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { IconButtonWithSpinnerComponent } from '@alfa-client/ui';
import { AssignUserProfileButtonContainerComponent } from '@alfa-client/user-profile';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { CreateWiedervorlageButtonContainerComponent } from '@alfa-client/wiedervorlage';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { AbschliessenButtonComponent } from '../../../buttons/abschliessen-button/abschliessen-button.component';
import { AnnehmenButtonComponent } from '../../../buttons/annehmen-button/annehmen-button.component';
import { BearbeitenButtonComponent } from '../../../buttons/bearbeiten-button/bearbeiten-button.component';
import { BescheidenButtonComponent } from '../../../buttons/bescheiden-button/bescheiden-button.component';
import { VerwerfenButtonComponent } from '../../../buttons/verwerfen-button/verwerfen-button.component';
import { WiedereroeffnenButtonComponent } from '../../../buttons/wiedereroeffnen-button/wiedereroeffnen-button.component';
import { ZurueckholenButtonComponent } from '../../../buttons/zurueckholen-button/zurueckholen-button.component';
import { ZurueckstellenButtonComponent } from '../../../buttons/zurueckstellen-button/zurueckstellen-button.component';
import { VorgangDetailActionButtonsComponent } from './vorgang-detail-action-buttons.component';

describe('VorgangDetailActionButtonsComponent', () => {
  let component: VorgangDetailActionButtonsComponent;
  let fixture: ComponentFixture<VorgangDetailActionButtonsComponent>;

  const loeschenAnfordernIconButtonContainer: string = getDataTestIdOf(
    'loeschen-anfordern-icon-button-container',
  );
  const createBescheidButtonContainer: string = getDataTestIdOf('create-bescheid-button-container');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailActionButtonsComponent,
        HasLinkPipe,
        MockComponent(IconButtonWithSpinnerComponent),
        MockComponent(AnnehmenButtonComponent),
        MockComponent(BearbeitenButtonComponent),
        MockComponent(VerwerfenButtonComponent),
        MockComponent(ZurueckholenButtonComponent),
        MockComponent(BescheidenButtonComponent),
        MockComponent(AbschliessenButtonComponent),
        MockComponent(ZurueckstellenButtonComponent),
        MockComponent(WiedereroeffnenButtonComponent),
        MockComponent(AssignUserProfileButtonContainerComponent),
        MockComponent(CreateWiedervorlageButtonContainerComponent),
        MockComponent(PostfachMailButtonContainerComponent),
        MockComponent(LoeschenAnfordernButtonContainerComponent),
        MockComponent(EndgueltigLoeschenButtonContainerComponent),
        MockComponent(LoeschAnforderungZuruecknehmenButtonContainerComponent),
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangDetailActionButtonsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('loeschenAnfordernIconButtonContainer', () => {
    it('should show if link is present', () => {
      component.vorgangWithEingang = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.LOESCHEN_ANFORDERN,
      ]);
      fixture.detectChanges();

      existsAsHtmlElement(fixture, loeschenAnfordernIconButtonContainer);
    });

    it('should hide if link is not present', () => {
      component.vorgangWithEingang = createVorgangWithEingangResource();
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, loeschenAnfordernIconButtonContainer);
    });
  });

  describe('create bescheid button', () => {
    it('should hide if link not exists', () => {
      component.vorgangWithEingang = createVorgangWithEingangResource();

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, createBescheidButtonContainer);
    });
  });
});
