/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ExpansionPanelComponent } from 'libs/ui/src/lib/ui/expansion-panel/expansion-panel.component';
import { MockComponent } from 'ng-mocks';
import {
  FormField,
  VorgangDetailFormDataTableComponent,
} from './vorgang-detail-form-data-table.component';

describe('VorgangDetailFormDataTableComponent', () => {
  let component: VorgangDetailFormDataTableComponent;
  let fixture: ComponentFixture<VorgangDetailFormDataTableComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [VorgangDetailFormDataTableComponent, MockComponent(ExpansionPanelComponent)],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangDetailFormDataTableComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('isExpandable', () => {
    describe('should return true', () => {
      it('on object with object as value', () => {
        const value: FormField = {
          value: {
            oneNode: {},
          },
        };

        const result: boolean = component.isExpandable(value);

        expect(result).toBeTruthy();
      });

      it('on object with array as value', () => {
        const value: FormField = {
          value: [
            {
              oneNode: {},
            },
          ],
        };

        const result: boolean = component.isExpandable(value);

        expect(result).toBeTruthy();
      });
    });

    describe('should return false', () => {
      it('on object with string as value', () => {
        const value: FormField = { value: 'StringValue' };

        const result: boolean = component.isExpandable(value);

        expect(result).toBeFalsy();
      });

      it('on object without value', () => {
        const result: boolean = component.isExpandable({ irgendwasAusserValue: 'zb' });

        expect(result).toBeFalsy();
      });
    });
  });

  describe('hasLabel', () => {
    it('should return true if node has label', () => {
      const value: FormField = { value: 'value', label: 'TestLabel' };

      const result: boolean = component.hasLabel(value);

      expect(result).toBeTruthy();
    });

    it('should return false if node has no label', () => {
      const value: FormField = { value: 'value' };

      const result: boolean = component.hasLabel(value);

      expect(result).toBeFalsy();
    });
  });

  describe('getFieldLabel', () => {
    it('should return field label from node with label', () => {
      const row = { key: 'label', value: { value: 'valueWithLabel', label: 'TestLabel' } };

      const label: string = component.getLabel(row);

      expect(label).toBe('TestLabel');
    });

    it('should return field key from node without label', () => {
      const row = { key: 'value', value: { value: 'simpleValue' } };

      const label: string = component.getLabel(row);

      expect(label).toBe(row.key);
    });
  });
});
