import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { OzgcloudIconComponent } from '@alfa-client/ui';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { OzgcloudSvgIconComponent } from 'libs/ui/src/lib/ui/ozgcloud-svgicon/ozgcloud-svgicon.component';
import { MockComponents } from 'ng-mocks';
import { WiedervorlageIconComponent } from './wiedervorlage-icon.component';

describe('WiedervorlageIconComponent', () => {
  let component: WiedervorlageIconComponent;
  let fixture: ComponentFixture<WiedervorlageIconComponent>;

  const wiedervorlageIconIsOverdue: string = getDataTestClassOf('wiedervorlage-icon-is-overdue');
  const wiedervorlageIconDefault: string = getDataTestClassOf('wiedervorlage-icon-default');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        WiedervorlageIconComponent,
        MatIcon,
        MockComponents(OzgcloudIconComponent),
        MockComponents(OzgcloudSvgIconComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(WiedervorlageIconComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('Create default Wiedervorlage Icon', () => {
    it('should show icon', () => {
      component.isOverdue = false;

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageIconDefault);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show icon', () => {
      component.isOverdue = true;

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageIconDefault);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('Create overdue Wiedervorlage Icon', () => {
    it('should show icon', () => {
      component.isOverdue = true;

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageIconIsOverdue);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show icon', () => {
      component.isOverdue = false;

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageIconIsOverdue);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });
});
