/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(CommandController.COMMANDS_PATH)
@RequiredArgsConstructor
public class CommandController {

	public static final String COMMANDS_PATH = "/api/commands"; // NOSONAR

	static final String PARAM_PENDING = "pending";
	static final String PARAM_VORGANG_ID = "vorgangId";

	private final CommandService service;
	private final CommandModelAssembler modelAssembler;

	@GetMapping("{commandId}")
	public EntityModel<Command> getById(@PathVariable String commandId) {
		return modelAssembler.toModel(service.getById(commandId));
	}

	@PatchMapping("{commandId}")
	public ResponseEntity<EntityModel<Command>> revoke(@PathVariable String commandId, @RequestBody StatusPatch patch) {
		if (patch.getStatus() == CommandStatus.REVOKED) {
			return ResponseEntity.ok(modelAssembler.toModel(service.revoke(commandId)));
		}
		return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
	}

	@GetMapping(params = { PARAM_PENDING, PARAM_VORGANG_ID })
	public CollectionModel<EntityModel<Command>> getPendingCommands(@RequestParam boolean pending, @RequestParam String vorgangId) {
		return modelAssembler.toCollectionModel(service.getPendingCommands(vorgangId));
	}

	public boolean existsPendingCommands(String relationId) {
		return service.existsPendingCommands(relationId);
	}

	@RestController
	@RequestMapping(CommandByRelationController.COMMAND_BY_RELATION_PATH)
	@RequiredArgsConstructor
	public static class CommandByRelationController {

		public static final String COMMAND_BY_RELATION_PATH = "/api/vorgangs/{vorgangId}/relations/{relationId}/{relationVersion}/commands"; // NOSONAR

		private final CommandService service;

		@PostMapping
		public ResponseEntity<Void> createCommand(@PathVariable String vorgangId, @PathVariable String relationId,
				@PathVariable long relationVersion, @RequestBody CreateCommand command) {
			command = command.toBuilder().vorgangId(vorgangId).relationId(relationId).build();

			var created = service.createCommand(command, relationVersion);

			return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
		}

		public Command createCommand(CreateCommand command) {
			return service.createCommand(command);
		}
	}
}