/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.nachrichten.postfach.GrpcFindPostfachMailRequest;
import de.ozgcloud.nachrichten.postfach.GrpcFindPostfachMailsRequest;
import de.ozgcloud.nachrichten.postfach.GrpcGetPostfachConfigRequest;
import de.ozgcloud.nachrichten.postfach.GrpcResendPostfachMailRequest;
import de.ozgcloud.nachrichten.postfach.PostfachServiceGrpc.PostfachServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class PostfachMailRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private PostfachServiceBlockingStub serviceStub;
	@Autowired
	private ContextService contextService;
	@Autowired
	private PostfachMailMapper postfachNachrichtMapper;
	@Autowired
	private PostfachConfigMapper postfachConfigMapper;

	public Stream<PostfachMail> findPostfachMails(String vorgangId) {
		var response = serviceStub.findPostfachMails(buildFindPostfachMailsRequest(vorgangId));

		return response.getMailsList().stream().map(postfachNachrichtMapper::toPostfachMail);
	}

	private GrpcFindPostfachMailsRequest buildFindPostfachMailsRequest(String vorgangId) {
		return GrpcFindPostfachMailsRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setContext(contextService.createCallContext())
				.build();
	}

	public Optional<PostfachMail> findById(PostfachNachrichtId nachrichtId) {
		var response = serviceStub.findPostfachMail(buildFindPostfachMailRequest(nachrichtId));

		return Optional.ofNullable(response.getNachricht()).map(postfachNachrichtMapper::toPostfachMail);
	}

	private GrpcFindPostfachMailRequest buildFindPostfachMailRequest(PostfachNachrichtId nachrichtId) {
		return GrpcFindPostfachMailRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setNachrichtId(nachrichtId.toString())
				.build();
	}

	public void resendPostfachMail(String commandId, String postfachMailId) {
		serviceStub.resendPostfachMail(buildResendPostfachMailRequest(commandId, postfachMailId));
	}

	GrpcResendPostfachMailRequest buildResendPostfachMailRequest(String commandId, String postfachMailId) {
		return GrpcResendPostfachMailRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setPostfachMailId(postfachMailId)
				.setCommandId(commandId)
				.build();
	}

	public PostfachConfigGroup getPostfachConfig() {
		var response = serviceStub.getPostfachConfig(buildGetPostfachConfigRequest());
		return postfachConfigMapper.toPostfachConfigGroup(response);
	}

	private GrpcGetPostfachConfigRequest buildGetPostfachConfigRequest() {
		return GrpcGetPostfachConfigRequest.newBuilder().build();
	}
}