package de.ozgcloud.alfa.bescheid;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

class DocumentControllerTest {

	@Mock
	private DocumentService documentService;
	@Mock
	private DocumentModelAssembler assembler;
	@Spy
	@InjectMocks
	private DocumentController controller;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetDocument {

		private final Document document = DocumentTestFactory.create();

		@BeforeEach
		void init() {
			when(documentService.getDocument(DocumentTestFactory.ID)).thenReturn(document);
		}

		@Test
		void shouldGetDocument() throws Exception {
			doRequest();

			verify(documentService).getDocument(DocumentTestFactory.ID);
		}

		@Test
		void shouldCallAssembler() throws Exception {
			doRequest();

			verify(assembler).toModel(document);
		}

		@Test
		void shouldReturnValue() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(DocumentController.PATH + "/{documentId}", DocumentTestFactory.ID));
		}
	}
}
