/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.jwt.Jwt;

class DownloadAuthenticationHandlerTest {

	private DownloadAuthenticationHandler downloadAuthorizationHandler = new DownloadAuthenticationHandler();

	@Nested
	class TestAuthorizationKeycloak {

		@Mock
		private Jwt jwt;

		private Authentication authentication = mock(Authentication.class);

		@BeforeEach
		void init() {
			when(authentication.isAuthenticated()).thenReturn(Boolean.TRUE);
			when(authentication.getPrincipal()).thenReturn(jwt);
		}

		@Test
		void shouldAuthenticate() {
			var check = downloadAuthorizationHandler.check(FileId.createNew(), authentication);

			assertThat(check).isTrue();
		}
	}

	@Nested
	class TestAuthorizationUsernamePassword {
		private FileId fileId = FileId.createNew();

		private Authentication authentication = mock(UsernamePasswordAuthenticationToken.class);

		@BeforeEach
		void init() {
			when(authentication.isAuthenticated()).thenReturn(Boolean.TRUE);
			when(authentication.getPrincipal()).thenReturn(DownloadAlfaUserTestFactory.createBuilder().fileId(fileId).build());
		}

		@Test
		void shouldAuthenticate() {
			var check = downloadAuthorizationHandler.check(fileId, authentication);

			assertThat(check).isTrue();
		}

		@Test
		void shouldNotAuthenticateWrongFileId() {
			var check = downloadAuthorizationHandler.check(FileId.createNew(), authentication);

			assertThat(check).isFalse();
		}

		@Test
		void shouldNotAuthenticateNoFileId() {
			var check = downloadAuthorizationHandler.check(null, authentication);

			assertThat(check).isFalse();
		}

		@Test
		void shouldNotAuthenticate() {
			when(authentication.isAuthenticated()).thenReturn(Boolean.FALSE);

			var check = downloadAuthorizationHandler.check(fileId, authentication);

			assertThat(check).isFalse();
		}
	}
}
