package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

class VorgangWithEingangTest {

	@Test
	void shouldGetOrganisationseinheitWhenSet() {
		VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		String organisationseinheitenID = vorgang.getOrganisationseinheitenID();

		assertThat(organisationseinheitenID).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
	}

	@Test
	void shouldNotGetOrganisationseinheitWithEingangUnset() {
		VorgangWithEingang vorgang = createVorgangWithNullEingang();

		String organisationseinheitenID = vorgang.getOrganisationseinheitenID();

		assertThat(organisationseinheitenID).isNull();
	}

	@Test
	void shouldNotGetOrganisationseinheitWithZustaendigeStelleUnset() {
		VorgangWithEingang vorgang = createVorgangWithNullZustaendigeStelle();

		String organisationseinheitenID = vorgang.getOrganisationseinheitenID();

		assertThat(organisationseinheitenID).isNull();
	}

	@Test
	void shouldNotGetOrganisationseinheitWithOrganisationseinheitUnset() {
		VorgangWithEingang vorgang = createVorgangWithNullOrganisationseinheit();

		String organisationseinheitenID = vorgang.getOrganisationseinheitenID();

		assertThat(organisationseinheitenID).isNull();
	}

	private VorgangWithEingang createVorgangWithNullEingang() {
		return createVorgangWithEingang(null);
	}

	private VorgangWithEingang createVorgangWithNullZustaendigeStelle() {
		Eingang eingang = EingangTestFactory.createBuilder().zustaendigeStelle(null).build();

		return createVorgangWithEingang(eingang);
	}

	private VorgangWithEingang createVorgangWithNullOrganisationseinheit() {
		ZustaendigeStelle zustaendigeStelle = ZustaendigeStelle.builder().organisationseinheitenId(null).build();
		Eingang eingang = EingangTestFactory.createBuilder().zustaendigeStelle(zustaendigeStelle).build();

		return createVorgangWithEingang(eingang);
	}

	private VorgangWithEingang createVorgangWithEingang(Eingang eingang) {
		return VorgangWithEingangTestFactory.createBuilder().eingang(eingang).build();
	}

}