/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

@WithMockUser
@AutoConfigureMockMvc
@SpringBootTest
public class WiedervorlageControllerITCase {

	@MockBean
	private WiedervorlageService service;
	@MockBean
	private VorgangController vorgangController;
	@Autowired
	private MockMvc mockMvc;

	@WithMockUser
	@Nested
	class TestGetById {

		@BeforeEach
		void beforeEach() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(VorgangWithEingangTestFactory.create());
		}

		@WithMockUser(roles = { UserRole.EINHEITLICHER_ANSPRECHPARTNER })
		@Nested
		class OnRoleEinheitlicherAnsprechpartner {

			@Test
			void shouldReturnForbidden() throws Exception {
				callEndpoint().andExpect(status().isForbidden());
			}
		}

		@WithMockUser(roles = UserRole.VERWALTUNG_POSTSTELLE)
		@Nested
		class OnRoleVerwaltungPoststelle {

			@Test
			void shouldReturnForbidden() throws Exception {
				callEndpoint().andExpect(status().isForbidden());
			}
		}

		@WithMockUser(roles = UserRole.VERWALTUNG_USER)
		@Nested
		class OnRoleVerwaltungUser {

			@BeforeEach
			void mockController() {
				when(service.getById(any())).thenReturn(WiedervorlageTestFactory.create());
			}

			@Test
			void shouldReturnStatusOk() throws Exception {
				callEndpoint().andExpect(status().isOk());
			}

			@Test
			void shouldReturnResult() throws Exception {
				callEndpoint().andExpect(jsonPath("$._links.self").exists())
						.andExpect(jsonPath("$._links.createdBy").exists())
						.andExpect(jsonPath("$.createdAt").value(WiedervorlageTestFactory.CREATED_AT_STR))
						.andExpect(jsonPath("$.frist").value(WiedervorlageTestFactory.FRIST_STR));
			}
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(WiedervorlageController.WIEDERVORLAGE_PATH + "/" + WiedervorlageTestFactory.ID));
		}
	}
}