package de.ozgcloud.alfa.export;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.mockito.InjectMocks;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class ExportVorgangProcessorTest {

	@InjectMocks
	private ExportVorgangProcessor processor;

	private final UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

	@Nested
	class TestProcess {

		@DisplayName("Export link")
		@Nested
		class TestExportLink {

			@BeforeEach
			void init() {
				initUserProfileUrlProvider(urlProvider);
			}

			@Test
			void shouldAddLinkWhenVorgangIsAbgeschlossen() {
				var vorgang = vorgangInStatus(VorgangStatus.ABGESCHLOSSEN);

				var model = processor.process(EntityModel.of(vorgang));

				assertThat(model.getLink(ExportVorgangProcessor.REL_EXPORT)).isPresent().get().extracting(Link::getHref)
						.isEqualTo(ExportVorgangController.PATH + "/" + VorgangHeaderTestFactory.ID);
			}

			@ParameterizedTest
			@EnumSource(mode = EnumSource.Mode.EXCLUDE, names = "ABGESCHLOSSEN")
			void shouldNotAddLinkWhenVorgangIsNotAbgeschlossen(VorgangStatus status) {
				var vorgang = vorgangInStatus(status);

				var model = processor.process(EntityModel.of(vorgang));

				assertThat(model.getLink(ExportVorgangProcessor.REL_EXPORT)).isEmpty();
			}

			private VorgangWithEingang vorgangInStatus(VorgangStatus status) {
				return VorgangWithEingangTestFactory.createBuilder().status(status).build();
			}
		}

	}

}