{{/* vim: set filetype=mustache: */}}

{{/* error check 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec) */}}
{{/* Namespace */}}
{{- define "app.namespace" -}}
{{- if gt (len (.Release.Namespace)) 63 -}}
{{- fail (printf ".Release.Namespace %s ist zu lang (max. 63 Zeichen)" .Release.Namespace) -}}
{{- end -}}
{{ printf "%s" .Release.Namespace }}
{{- end -}}

{{/* Chart: Name + Version */}}
{{- define "app.chart" -}}
{{- if gt (len (printf "%s-%s" .Chart.Name .Chart.Version)) 63 -}}
{{- fail (printf ".Chart.Name-.Chart.Version %s-%s ist zu lang (max. 63 Zeichen)" .Chart.Name .Chart.Version) -}}
{{- end -}}
{{ printf "%s-%s" .Chart.Name .Chart.Version }}
{{- end -}}

{{/* Managed-by -> On Helm, this value is always Helm */}}
{{- define "app.managedBy" -}}
{{- if gt (len (.Release.Service)) 63 -}}
{{- fail (printf ".Release.Service %s ist zu lang (max. 63 Zeichen)" .Release.Service) -}}
{{- end -}}
{{ printf "%s" .Release.Service }}
{{- end -}}

{{/* Default Labels: Helm recommended best-practice labels https://helm.sh/docs/chart_best_practices/labels/ */}}
{{- define "app.defaultLabels" }}
app.kubernetes.io/instance: alfa
app.kubernetes.io/managed-by: {{ include "app.managedBy" . }}
app.kubernetes.io/name: {{ .Release.Name }}
app.kubernetes.io/part-of: ozgcloud
app.kubernetes.io/version: {{ .Chart.Version }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
helm.sh/chart: {{ include "app.chart" . }}
{{- end -}}

{{- define "app.matchLabels" }}
app.kubernetes.io/name: {{ .Release.Name }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
{{- end -}}

{{- define "app.envSpringProfiles" }}
{{- if (.Values.env).overrideSpringProfiles -}}
{{ printf "%s" (.Values.env).overrideSpringProfiles }}
{{- else -}}
{{ printf "oc, %s" (include "app.ozgcloudEnvironment" . ) }}
{{- end -}}
{{- end -}}

{{- define "app.grpc_client_vorgang_manager_address" -}}
{{ printf "dns:///%s.%s:9090" ( coalesce .Values.vorgangManagerName "vorgang-manager" ) .Release.Namespace | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.grpc_client_user-manager_address" -}}
{{ printf "%s.%s:9000" ( coalesce .Values.usermanagerName "user-manager" ) .Release.Namespace }}
{{- end -}}

{{- define "app.spring_cloud_config_administration_address" -}}
{{ printf "http://%s.%s:8080" ( coalesce .Values.administrationName "administration" ) .Release.Namespace }}
{{- end -}}

{{- define "app.baseUrl" -}}
{{- required "baseUrl muss angegeben sein" .Values.baseUrl }}
{{- end -}}

{{- define "app.ozgcloud_user-manager_url" -}}
{{- if (.Values).usermanagerUrl -}}
{{ printf "%s" (.Values).usermanagerUrl }}
{{- else -}}
{{ printf "https://%s-%s.%s" (include "app.ozgcloudBezeichner" .) .Values.usermanagerName (include "app.baseUrl" .) }}
{{- end -}}
{{- end -}}

{{- define "app.ozgcloudBundesland" -}}
{{- required "Bundesland muss angegeben sein" (.Values.ozgcloud).bundesland }}
{{- end -}}

{{- define "app.ozgcloudBezeichner" -}}
{{- required "ozgcloud.bezeichner muss angegeben sein" (.Values.ozgcloud).bezeichner -}}
{{- if lt 46 (len (.Values.ozgcloud).bezeichner) -}}
{{ fail (printf "ozgcloud.bezeichner %s ist zu lang (max. 46 Zeichen)" (.Values.ozgcloud).bezeichner) }}
{{- end -}}
{{- end -}}

{{- define "app.ozgcloudEnvironment" -}}
{{- required "Environment muss angegeben sein" (.Values.ozgcloud).environment -}}
{{- end -}}

{{- define "app.ssoRealm" -}}
{{ printf "%s-%s-%s" (include "app.ozgcloudBundesland" .) ( include "app.ozgcloudBezeichner" . ) ( include "app.ozgcloudEnvironment" . ) | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.ssoClientName" -}}
{{- if (.Values.sso).client_name -}}
{{ printf "%s" (.Values.sso).client_name }}
{{- else -}}
{{ printf "alfa" }}
{{- end -}}
{{- end -}}

{{- define "app.ssoRealmDisplayName" -}}
{{- if (.Values.sso).displayName -}}
{{ printf "%s" (.Values.sso).displayName }}
{{- else if eq (include "app.ozgcloudEnvironment" . ) "prod" -}}
{{ printf "Realm für %s" (include "app.ozgcloudBezeichner" . | title ) }}
{{- else -}}
{{ printf "Realm für %s (%s)" (include "app.ozgcloudBezeichner" . | title ) (include "app.ozgcloudEnvironment" .) }}
{{- end -}}
{{- end -}}

{{- define "app.ssoServerUrl" -}}
{{- required "sso.serverUrl muss angegeben sein" (.Values.sso).serverUrl -}}
{{- end -}}

{{- define "app.baseDomain" -}}
{{- printf "%s.%s" (include "app.ozgcloudBezeichner" .) (include "app.baseUrl" .) }}
{{- end -}}

{{- define "app.generateKeycloakGroupRessourceName" -}}
{{- $kebabcaseGroupName := . | title | nospace | kebabcase }}
{{- $alphanumericGroupName :=  (include "app.replaceAllNonAlphanumeric" $kebabcaseGroupName ) }}
{{- printf "%s-keycloak-group"  ( $alphanumericGroupName | trunc 20) }}
{{- end -}}

{{- define "app.replaceAllNonAlphanumeric" -}}
{{- trimAll "-" ( regexReplaceAll "[^a-zA-Z0-9-]" . "" ) }}
{{- end -}}

{{- define "app.generateKeycloakUserRessourceName" -}}
{{- $lowerUserName := . | lower }}
{{- $alphanumericUserName :=  (include "app.replaceAllNonAlphanumeric" $lowerUserName ) }}
{{- printf "%s-keycloak-user"  ( $alphanumericUserName | trunc 20) }}
{{- end -}}

{{- define "app.generateKeycloakClientRessourceName" -}}
{{- $lowerClientName := . | lower }}
{{- $alphanumericClientName :=  (include "app.replaceAllNonAlphanumeric" $lowerClientName ) }}
{{- printf "%s-keycloak-client"  ( $alphanumericClientName | trunc 20) }}
{{- end -}}

{{- define "app.serviceAccountName" -}}
{{ printf "%s" ( (.Values.serviceAccount).name | default "alfa-service-account" ) }}
{{- end -}}

{{- define "app.getCustomList" -}}
{{- with (.Values.env).customList -}}
{{- if kindIs "map" . -}}
{{ include "app.dictToList" . }}
{{- else if kindIs "slice" . -}}
{{ . | toYaml }}
{{- end -}}
{{- end -}}
{{- end -}}

{{- define "app.dictToList" -}}
{{- $customList := list -}}
{{- range $key, $value := . -}}
{{- $customList = append $customList (dict "name" $key "value" $value) }}
{{- end -}}
{{- $customList | toYaml -}}
{{- end -}}